@@ -5,6 +5,7 @@ from _echopype_version import version as __version__  # noqa
 from . import calibrate
 from .convert.api import open_raw
 from .echodata.api import open_converted
+from .echodata.combine import combine_echodata
 from .process import Process  # noqa
 
-__all__ = ["open_raw", "open_converted", "calibrate"]
+__all__ = ["open_raw", "open_converted", "combine_echodata", "calibrate"]
