@@ -14,7 +14,7 @@ class CalibrateEK(CalibrateBase):
         self.env_params = dict.fromkeys(ENV_PARAMS)
         self.cal_params = dict.fromkeys(CAL_PARAMS["EK"])
 
-    def compute_range_meter(self, waveform_mode, tvg_correction_factor):
+    def compute_range_meter(self, waveform_mode, encode_mode="complex"):
         """
         Parameters
         ----------
@@ -30,42 +30,11 @@ class CalibrateEK(CalibrateBase):
         range_meter : xr.DataArray
             range in units meter
         """
-        if waveform_mode == "CW":
-            sample_thickness = (
-                self.echodata.beam["sample_interval"]
-                * self.env_params["sound_speed"]
-                / 2
-            )
-            # TODO: Check with the AFSC about the half sample difference
-            range_meter = (
-                self.echodata.beam.range_bin - tvg_correction_factor
-            ) * sample_thickness  # [frequency x range_bin]
-        elif waveform_mode == "BB":
-            # TODO: bug: right now only first ping_time has non-nan range
-            shift = self.echodata.beam[
-                "transmit_duration_nominal"
-            ]  # based on Lar Anderson's Matlab code
-            # TODO: once we allow putting in arbitrary sound_speed,
-            # change below to use linearly-interpolated values
-            range_meter = (
-                (
-                    self.echodata.beam.range_bin * self.echodata.beam["sample_interval"]
-                    - shift
-                )
-                * self.env_params["sound_speed"].squeeze()
-                / 2
-            )
-            # TODO: Lar Anderson's code include a slicing by minRange with a default of 0.02 m,
-            #  need to ask why and see if necessary here
-        else:
-            raise ValueError("Input waveform_mode not recognized!")
-
-        # make order of dims conform with the order of backscatter data
-        range_meter = range_meter.transpose("frequency", "ping_time", "range_bin")
-        range_meter = range_meter.where(range_meter > 0, 0)  # set negative ranges to 0
-        range_meter.name = "range"  # add name to facilitate xr.merge
-
-        self.range_meter = range_meter
+        self.range_meter = self.echodata.compute_range(
+            self.env_params,
+            ek_waveform_mode=waveform_mode,
+            ek_encode_mode=encode_mode,
+        )
 
     def _get_vend_cal_params_power(self, param):
         """Get cal parameters stored in the Vendor group.
@@ -79,7 +48,7 @@ class CalibrateEK(CalibrateBase):
         #  currently this has only been tested with EK60 data
         ds_vend = self.echodata.vendor
 
-        if param not in ds_vend:
+        if ds_vend is None or param not in ds_vend:
             return None
 
         if param not in ["sa_correction", "gain_correction"]:
@@ -101,36 +70,65 @@ class CalibrateEK(CalibrateBase):
                 dim="ping_time", how="any", subset=["transmit_duration_nominal"]
             )
 
+        if self.echodata.beam_power is not None:
+            beam = self.echodata.beam_power
+        else:
+            beam = self.echodata.beam
+
+            # indexes of frequencies that are for power, not complex
+        relevant_indexes = np.where(np.isin(ds_vend["frequency"], beam["frequency"]))[0]
+
+        unique_pulse_length = np.unique(beam["transmit_duration_nominal"], axis=1)
+
+        pulse_length = ds_vend["pulse_length"][relevant_indexes]
+
         # Find index with correct pulse length
-        unique_pulse_length = np.unique(
-            self.echodata.beam["transmit_duration_nominal"], axis=1
-        )
-        idx_wanted = np.abs(ds_vend["pulse_length"] - unique_pulse_length).argmin(
+        idx_wanted = np.abs(pulse_length - unique_pulse_length).argmin(
             dim="pulse_length_bin"
         )
 
-        return ds_vend[param].sel(pulse_length_bin=idx_wanted).drop("pulse_length_bin")
+        return (
+            ds_vend[param]
+            .isel(pulse_length_bin=idx_wanted, frequency=relevant_indexes)
+            .drop("pulse_length_bin")
+        )
 
-    def get_cal_params(self, cal_params):
+    def get_cal_params(self, cal_params, waveform_mode=None, encode_mode="complex"):
         """Get cal params using user inputs or values from data file.
 
         Parameters
         ----------
         cal_params : dict
         """
+
+        if (
+            encode_mode == "power"
+            and waveform_mode == "CW"
+            and self.echodata.beam_power is not None
+        ):
+            beam = self.echodata.beam_power
+        else:
+            beam = self.echodata.beam
+
         # Params from the Vendor group
-        params_from_vend = ["sa_correction", "gain_correction"]
-        for p in params_from_vend:
-            # substitute if None in user input
-            self.cal_params[p] = (
-                cal_params[p] if p in cal_params else self._get_vend_cal_params_power(p)
-            )
+        # only execute this if cw and power
+        if waveform_mode == "CW" and (
+            self.echodata.beam_power is not None or "quadrant" not in self.echodata.beam
+        ):
+            params_from_vend = ["sa_correction", "gain_correction"]
+            for p in params_from_vend:
+                # substitute if None in user input
+                self.cal_params[p] = (
+                    cal_params[p]
+                    if p in cal_params
+                    else self._get_vend_cal_params_power(p)
+                )
 
         # Other params
         self.cal_params["equivalent_beam_angle"] = (
             cal_params["equivalent_beam_angle"]
             if "equivalent_beam_angle" in cal_params
-            else self.echodata.beam["equivalent_beam_angle"]
+            else beam["equivalent_beam_angle"]
         )
 
     def _cal_power(self, cal_type, use_beam_power=False):
@@ -221,10 +219,10 @@ class CalibrateEK60(CalibrateEK):
         self.get_env_params(env_params)
         if cal_params is None:
             cal_params = {}
-        self.get_cal_params(cal_params)
+        self.get_cal_params(cal_params, waveform_mode="CW", encode_mode="power")
 
         # default to CW mode recorded as power samples
-        self.compute_range_meter(waveform_mode="CW", tvg_correction_factor=2)
+        self.compute_range_meter(waveform_mode="CW", encode_mode="power")
 
     def get_env_params(self, env_params, **kwargs):
         """Get env params using user inputs or values from data file.
@@ -281,7 +279,7 @@ class CalibrateEK80(CalibrateEK):
     z_et = 75
     z_er = 1000
 
-    def __init__(self, echodata, env_params, cal_params, waveform_mode):
+    def __init__(self, echodata, env_params, cal_params, waveform_mode, encode_mode):
         super().__init__(echodata)
 
         # initialize env and cal params
@@ -294,15 +292,19 @@ class CalibrateEK80(CalibrateEK):
         # load env and cal parameters
         if env_params is None:
             env_params = {}
-        self.get_env_params(env_params, waveform_mode=waveform_mode)
+        self.get_env_params(
+            env_params, waveform_mode=waveform_mode, encode_mode=encode_mode
+        )
         if cal_params is None:
             cal_params = {}
-        self.get_cal_params(cal_params)
+        self.get_cal_params(
+            cal_params, waveform_mode=waveform_mode, encode_mode=encode_mode
+        )
 
         # self.range_meter computed under self._compute_cal()
         # because the implementation is different depending on waveform_mode and encode_mode
 
-    def get_env_params(self, env_params, waveform_mode=None):
+    def get_env_params(self, env_params, waveform_mode=None, encode_mode="complex"):
         """Get env params using user inputs or values from data file.
 
         EK80 file by default contains sound speed, temperature, depth, salinity, and acidity,
@@ -316,15 +318,26 @@ class CalibrateEK80(CalibrateEK):
         waveform_mode : str
             ``CW`` for CW-mode samples, either recorded as complex or power samples
             ``BB`` for BB-mode samples, recorded as complex samples
+        encode_mode : str
+            EK80 data can be encoded as complex samples or power samples.
+            Use ``complex`` to compute Sv from only complex samples,
+            and ``power`` to compute Sv from only power samples.
         """
+
+        if (
+            encode_mode == "power"
+            and waveform_mode == "CW"
+            and self.echodata.beam_power is not None
+        ):
+            beam = self.echodata.beam_power
+        else:
+            beam = self.echodata.beam
+
         # Use center frequency if in BB mode, else use nominal channel frequency
         if waveform_mode == "BB":
-            freq = (
-                self.echodata.beam["frequency_start"]
-                + self.echodata.beam["frequency_end"]
-            ) / 2
+            freq = (beam["frequency_start"] + beam["frequency_end"]) / 2
         else:
-            freq = self.echodata.beam["frequency"]
+            freq = beam["frequency"]
 
         # Re-calculate environment parameters if user supply all env variables
         if (
@@ -762,6 +775,9 @@ class CalibrateEK80(CalibrateEK):
 
         # Warn user about additional data in the raw file if another type exists
         if self.echodata.beam_power is not None:  # both power and complex samples exist
+            if encode_mode == "complex" and waveform_mode == "CW":
+                raise ValueError("file does not contain CW complex samples")
+
             if encode_mode == "power":
                 use_beam_power = True  # switch source of backscatter data
                 print(
@@ -788,11 +804,11 @@ class CalibrateEK80(CalibrateEK):
         # Compute Sv
         if flag_complex:
             self.compute_range_meter(
-                waveform_mode=waveform_mode, tvg_correction_factor=0
+                waveform_mode=waveform_mode, encode_mode=encode_mode
             )
             ds_cal = self._cal_complex(cal_type=cal_type, waveform_mode=waveform_mode)
         else:
-            self.compute_range_meter(waveform_mode="CW", tvg_correction_factor=0)
+            self.compute_range_meter(waveform_mode="CW", encode_mode=encode_mode)
             ds_cal = self._cal_power(cal_type=cal_type, use_beam_power=use_beam_power)
 
         return ds_cal
